/*
 * Simplifier.cpp
 *
 *  Created on: Jan 5, 2010
 *      Author: zielke
 */

#include "DMRPSolver.hpp"
#include "Simplifier.hpp"
#include <vector>
#include <algorithm>
#include <queue>
#include <string.h>
#include <sstream>

using namespace std;

/**
 * simple constructor
 */
Simplifier::Simplifier(BasicSolver* solver, Statistics* stats, bool simpblocked, bool simpvarelim, bool printModel)
{
	mySolver = solver;
	myStats = stats;

	//add the dummy element
	reserveSpaceForClauses(1);

	void* mem								= malloc(sizeof(Clause) + sizeof(Literal)*1);
	if(mem == NULL){
		fprintf(stderr,"\nc !!!OUT OF MEMORY - Try to Fix!!!\n");
		exit(EXIT_FAILURE);
		return;
	}
	Clause* cls 							= new (mem) Clause(1, false, 0);
	constraints[0] = cls;

	activeClauses = 0;
	varAssignments = 0;

	duringSimp = false;

	blockingClauses = simpblocked;

	varelim = simpvarelim;

	amountOfBlockingClauses = 0;

	printAssignmentModel = printModel;
}

/**
 * Method that exports the simplified constraints to the solver
 */
void Simplifier::exportConstraintsToSolver()
{
	for( uint i = 1 ; i<constraints.size() ; i++ )
	{
		if(!(*constraints[i]).null())
		{
			if(!(*mySolver).addClauseFromSimplifier(constraints[i]))
			{
				 fprintf(stderr,"c Solved by Preprocessor");
				 (*mySolver).setInvalid(); return;
			}
		}

		if(!printAssignmentModel)
			if(constraints[i] != constraints[0])
				free(constraints[i]);
	}

	if(!printAssignmentModel)
		free(constraints[0]);

	if(status == TRUE_TrBool)
	{
		for(uint i=0;i<varAssignment.size();i++)
		{
			if(varAssignment[i] != UNKNOWN_TrBool)
			{
				vector<Literal> lits;
				if(varAssignment[i] == TRUE_TrBool)
					lits.push_back(Variable(i).toPosLit());
				else
					lits.push_back(Variable(i).toNegLit());

				(*mySolver).addClause(lits);
				return;
			}

		}
	}
}

/**
 * Method that reserves spaces on the disc for all data structures that handle Clauses
 */
void Simplifier::reserveSpaceForClauses(uint32_t count)
{
	count++;
	if(count >= constraints.size())
	{
		constraints.resize(count);
		isClauseTouched.ensureSize(count,true);
	}
}

/**
 * Method that reserves spaces on the disc for all data structures that handle Literals
 */
void Simplifier::reserveSpaceForVars(uint32_t count)
{
	//count++;
	if(count >= varAssignment.size())
	{
		varAssignment.resize(count, UNKNOWN_TrBool);
		modelAssignment.resize(count, UNKNOWN_TrBool);
		varEliminated.ensureSize(count, true);
		occur.resize(count*2);
	}
}

/**
 * The Clause was created earlier in BasicSolver,
 * here only the pointer is added
 */
bool Simplifier::addClause( Clause* myClause, uint32_t id )
{
	reserveSpaceForClauses(id);

	(*myClause).computeSignature();
	(*myClause).setID(id);

	constraints[id] = myClause;
	addClauseToOccur(myClause);

	activeClauses++;

	if((*myClause).size() == 1)
		if(!propagate((*myClause)[0]))
		{

			setSolvedStatus(FALSE_TrBool);
			return false;
		}

	return true;
}

/**
 * The Clause is created during Simplification
 * so a lot more to do
 */
bool Simplifier::addClause( vector<Literal> lits, uint32_t id )
{
	reserveSpaceForClauses(id);

	//sort the input vector
	sort(lits.begin(), lits.end());

	reserveSpaceForVars(lits.back().toVar().idx());

	asrt(id < constraints.size());


	//-check for double literals and tautology check
	//-check the assignments of the variables
	//	-> eliminate negative variables from the clause
	//	-> if clause contains a positive assignment, do not add it to the database
	vector<Literal> checkedLits;
	for(uint i=0;i<lits.size();i++)
	{
		if(i < lits.size() - 1)
		{
			//double literal check
			if(lits[i] == lits[i+1])
				continue;
			//tautology check
			if(lits[i].flipped() == lits[i+1])
			{
				constraints[id] = constraints[0]; //constraints[0] is dummy element - marks deleted clause
				return true;
			}
		}

		//check the assignments
		if(varAssignment[lits[i].toVar().idx()] == UNKNOWN_TrBool)
			checkedLits.push_back(lits[i]);
		else
		{
			if(lits[i].isPos())
			{
				if(varAssignment[lits[i].toVar().idx()] == TRUE_TrBool)
				{
					constraints[id] = constraints[0]; //constraints[0] is dummy element - marks deleted clause
					return true;
				}
			}else
			{
				if(varAssignment[lits[i].toVar().idx()] == FALSE_TrBool)
				{
					constraints[id] = constraints[0]; //constraints[0] is dummy element - marks deleted clause
					return true;
				}
			}
		}
	}

	if(checkedLits.size() == 0)
	{

		setSolvedStatus(FALSE_TrBool);
		return false;
	}

	if(checkedLits.size() == 1)
	{
		constraints[id] = constraints[0];
		return propagate(checkedLits[0]);
	}

	vector<Literal>::iterator lbeg = checkedLits.begin();
	vector<Literal>::iterator lend = checkedLits.end();

	reserveSpaceForVars(checkedLits[checkedLits.size()-1].toVar().idx());

	uint32_t cl_sz							= lend-lbeg;
	asrt(cl_sz <= Clause::MAX_SIZE);

	void* mem								= malloc(sizeof(Clause) + sizeof(Literal)*cl_sz);
	if(mem == NULL){
		fprintf(stderr,"\nc !!!OUT OF MEMORY - Try to Fix!!!\n");
		exit(EXIT_FAILURE);
		return false;
	}
	Clause* cls 							= new (mem) Clause(cl_sz, false, id);

	for(uint32_t i = 0;lbeg != lend;++lbeg) {(*cls)[i++] = *lbeg;}

	(*cls).computeSignature();

	constraints[id] = cls;
	addClauseToOccur(cls);

	activeClauses++;
	return true;
}

/**
 * Method adds the pointer to the Clause to all occurrence vectors of the containing Literals
 */
void Simplifier::addClauseToOccur(Clause* myClause)
{
	for(uint i=0 ; i < (*myClause).size() ;i++)
		occur[(*myClause)[i].idx()].push_back(myClause);
}


/**
 * The name of the method says enough...
 */
bool Simplifier::addSetOfClausesduringElim(Variable toDel, vector<Clause*>& toAdd)
{
	vector<Clause*>&   poss = occur[toDel.toPosLit().idx()];
	vector<Clause*>&   negs = occur[toDel.toNegLit().idx()];

	uint curClauseID = constraints.size();

	reserveSpaceForClauses(curClauseID + toAdd.size() - 1);

	uint i=0;
	for(vector<Clause*>::iterator it = toAdd.begin();it != toAdd.end() ; it++)
	{
		if(!addClause(*it, curClauseID+i))
			return false;

		touch(constraints[curClauseID+i]);
		i++;
	}

	for(vector<Clause*>::iterator it = poss.begin();it != poss.end() ; it++)
		deleteClause(*it);
	for(vector<Clause*>::iterator it = negs.begin();it != negs.end() ; it++)
		deleteClause(*it);

	return true;
}

/**
 * Method deletes a whole clause from database
 * for instance due to being a tautology
 */
void Simplifier::deleteClause(Clause* myClause)
{
	if(!(*myClause).null())
	{
		(*myClause).del();
		decreaseActiveClauses();
	}
}


bool Simplifier::propagate(Literal x)
{
	//test whether a unit propagation with the literal was already done and handle the cases
	if(varAssignment[x.toVar().idx()] != UNKNOWN_TrBool)
	{
		if(x.isPos())
		{
			if(varAssignment[x.toVar().idx()] == TRUE_TrBool)
				return true;
			else
			{

				if(duringSimp)
					setSolvedStatus(FALSE_TrBool);
				return false;
			}
		}else
		{
			if(varAssignment[x.toVar().idx()] == FALSE_TrBool)
				return true;
			else
			{

				if(duringSimp)
					setSolvedStatus(FALSE_TrBool);
				return false;
			}
		}
	}
	if(x.isPos())
		varAssignment[x.toVar().idx()] = TRUE_TrBool;
	else
		varAssignment[x.toVar().idx()] = FALSE_TrBool;

	//all clauses where x occurs can be 'deleted' from the database
	//all clauses where ~x occurs can be strengthened -> delete ~x from the clause -> test whether a new unit arose

	vector<Clause*>& posX = occur[x.idx()];
	vector<Clause*>& negX = occur[x.flipped().idx()];

	//handle positive occurrences
	for(vector<Clause*>::iterator pIT = posX.begin(); pIT != posX.end(); pIT++)
	{
		if((**pIT).null())
			continue;

		(**pIT).del();

		if(duringSimp)
			decreaseActiveClauses();
	}

	//handle negative occurrences
	for(vector<Clause*>::iterator nIT = negX.begin(); nIT != negX.end(); nIT++)
	{
		if((**nIT).null())
			continue;

		if((**nIT).size() == 1)
		{
			if(duringSimp)
				setSolvedStatus(FALSE_TrBool);
			return false;
		}

		//do not use the method strengthen clause,
		//because of the overhead produced when single deleting the clause pointer from occurrence-lists
		(**nIT).eliminateLiteral(x.flipped());

		//add the clause to touched
		touch(*nIT);

		//propagate further
		if((**nIT).size() == 1)
		{
//			cout << "new unit" << endl;
			Literal toAdd = (**nIT)[0];
			if(!propagate(toAdd))
				return false;
		}
	}
	//clear the occurrence-lists from x and ~x
	posX.clear();
	negX.clear();
	return true;
}

/**
 * Method checks if the signatures of 2 Clauses reflect a possible subset case
 */
bool Simplifier::subsetTest(uint32_t signature, uint32_t otherSignature)
{
	return (otherSignature & (~signature)) == 0;
}

/**
 * Method checks via full expensive test, if the Clause with 'otherLits' Literals is a
 * subset of the Clause with 'lits' Literals
 * -> assumptions:
 * 		the vector<Lit>'s are sorted
 * 		no double Lit's in the Clauses/vectors
 */
bool Simplifier::subsetTest(vector<Literal> lits, vector<Literal> otherLits)
{

	vector<Literal>::iterator itLits = lits.begin();
	vector<Literal>::iterator itOtherLits = otherLits.begin();

	while(true)
	{
		//checked all otherLits elements
		if(itOtherLits == otherLits.end())
			return true;

		//checked not all otherLits elements, but all elements of lits
		if(itLits == lits.end())
			return false;

		if((*itLits) == (*itOtherLits))
		{
			itLits++;
			itOtherLits++;
		}else
		{
			if((*itLits) < (*itOtherLits))
				itLits++;
			else
				return false;
		}
	}
	return false;
}

/**
 * Method identifies the clauses that are subsumed by 'myClause'
 */
void Simplifier::findSubsumedClauses(Clause *myClause, vector<Clause*>& subsumedClauses)
{
	subsumedClauses.clear();

	if((*myClause).null())
		return;

	//search the lit in myClause with minimal occurrences
	int minimalOccurencesIndex = 0;

	for(uint i=1;i<(*myClause).size();i++)
	{
		//here the size() is not really the number of elements in the vector, because 'deleted' clauses are contained as well
		if(occur[((*myClause)[i]).idx()].size() < occur[((*myClause)[minimalOccurencesIndex]).idx()].size())
			minimalOccurencesIndex = i;
	}
	//take as candidates the clauses that have the same lit
	asrt(((*myClause)[minimalOccurencesIndex]).idx() < occur.size());
	vector<Clause*>& candidates = occur[((*myClause)[minimalOccurencesIndex]).idx()];

	//for every candidate
	for(vector<Clause*>::iterator it = candidates.begin(); it != candidates.end() ; it++)
	{
		if((**it).null())
			continue;
		/**
		 * 1. pointer inequality (ID-test)
		 * 2. size of subset has to be smaller than main set
		 * 3. possible subset-test via signatures
		 * 4. expensive subset-test
		 * -> if all true: found subsumed clause
		 */
		if(	((*myClause).getID() != (**it).getID())								&&
			((*myClause).size() < (**it).size())								&&
			(subsetTest((**it).getSignature(),(*myClause).getSignature()))		&&
			(subsetTest((**it).getAllLits(),  (*myClause).getAllLits()))
		  )
			subsumedClauses.push_back(*it);
	}
}

/**
 * Method that finds all Clauses, that can be self subsumed via myClause
 * e.g.   (-x a b) ^ (x a) => (a b) ^ (x a)
 * - touches the found clauses
 */
bool Simplifier::selfSubsumeVariant(Clause* myClause)
{
	vector<Clause*> subsumed;
	subsumed.resize(1);

	for(uint i=0;i<(*myClause).size();i++)
	{
		//negate the Literal
		(*myClause).flipLiteral(i);

		findSubsumedClauses(myClause, subsumed);

		//enqueue the strengthened clauses for further selfSubsume-Tests
		for(vector<Clause*>::iterator it = subsumed.begin(); it != subsumed.end();it++)
		{
			//strengthen the subsumed clauses:
			if(!strengthenClause(*it, (*myClause)[i]))
				return false;
		}
		//clear the vector for the next iteration
		subsumed.clear();

		//flip the Literal to its origin
		(*myClause).flipLiteral(i);
	}
	return true;
}

/**
 * Method strengthen a clause by eliminating a Literal
 */
bool Simplifier::strengthenClause(Clause* strClause, Literal toElim)
{
//	print_out("#%u |",(*strClause).getID());

	removeClauseFromOccur(strClause, toElim);
	(*strClause).eliminateLiteral(toElim);

	//may add it to propagation list, if a unit is built
	if((*strClause).size() == 1)
	{
		if(!propagate((*strClause)[0]))
		{

			setSolvedStatus(FALSE_TrBool);
			return false;
		}

	}else if((*strClause).size() == 0)
	{

		setSolvedStatus(FALSE_TrBool);
		return false;
	}
	//add it to the touched list
	touch(strClause);
	return true;
}

/**
 * Method looks for clauses, that are subsumed by myClause and deletes them form the database
 */
void Simplifier::eraseSubsumed(Clause* myClause)
{
	if((*myClause).null())
		return;

	vector<Clause*> subsumed;
	subsumed.resize(0);

	findSubsumedClauses(myClause, subsumed);

	for(vector<Clause*>::iterator myIT = subsumed.begin();myIT != subsumed.end();myIT++)
	{
		(**myIT).del();
		decreaseActiveClauses();
	}
}

/**
 * Method edits the occur-list
 * Helping method 'strenthenClause'
 */
void Simplifier::removeClauseFromOccur(Clause* myClause, Literal occurLit)
{
	vector<Clause*>& litsOccuring = occur[occurLit.idx()];
	vector<Clause*>::iterator it = litsOccuring.begin();
	while(*it != myClause)
		it++;

	(*it) = constraints[0];
}

/**
 * Method implements variable elimination with the help of computing the Cartesian product of positive occurrences and negative occurrences of the variable x
 * return: literal count
 */
int Simplifier::mergeClauses(Literal x, vector<Clause*>& pos, vector<Clause*>& neg, vector<Clause*>& merged)
{
	asrt(x.isPos());

	int literals = 0;

	vector<Literal> posLits;
	vector<Literal> negLits;
	vector<Literal> mergedLits;

	bool isTautology;

	//computing the Cartesian product
	for(vector<Clause*>::iterator posIT = pos.begin(); posIT != pos.end(); posIT++)
	{
		if((**posIT).null())
			continue;

		for(vector<Clause*>::iterator negIT = neg.begin(); negIT != neg.end(); negIT++)
		{
			if((**negIT).null())
				continue;

			posLits = (**posIT).getAllLits();
			negLits = (**negIT).getAllLits();

			mergedLits.clear();

			isTautology = false;

			//use the sortness of both input vectors
			vector<Literal>::iterator posLitIT = posLits.begin();
			vector<Literal>::iterator negLitIT = negLits.begin();

			while((posLitIT != posLits.end()) || (negLitIT != negLits.end()))
			{
				//if both have not reached their end
				if((posLitIT != posLits.end()) && (negLitIT != negLits.end()))
				{
					if((*posLitIT).flipped() == (*negLitIT))
					{
						if((*posLitIT) == x)
						{
							posLitIT++;
							negLitIT++;
							continue;
						}else{
							isTautology = true;
							break;
						}
					}

					if( (*posLitIT) == (*negLitIT) )
					{
						//add only once
						if(!((*posLitIT) == x))
							mergedLits.push_back((*posLitIT));

						posLitIT++;
						negLitIT++;
					}else if( (*posLitIT) < (*negLitIT) )
					{
						if(!((*posLitIT) == x))
							mergedLits.push_back((*posLitIT));

						posLitIT++;
					}else
					{
						asrt( (*negLitIT) < (*posLitIT) );
						if(!((*negLitIT) == x.flipped()))
							mergedLits.push_back((*negLitIT));

						negLitIT++;
					}

				}else{
					if(posLitIT == posLits.end())
					{
						//flush negLits into mergedLits
						while(negLitIT != negLits.end())
						{
							if(!((*negLitIT) == x.flipped()))
								mergedLits.push_back((*negLitIT));

							negLitIT++;
						}

					}else
					{
						asrt(negLitIT == negLits.end());
						//flush posLits into mergedLits
						while(posLitIT != posLits.end())
						{
							if(!((*posLitIT) == x))
								mergedLits.push_back((*posLitIT));

							posLitIT++;
						}

					}

				}


			}

			if(!isTautology)
			{

				//id = 0 to mark, that the clause is only a candidate to add to constraints
				vector<Literal>::iterator lbeg = mergedLits.begin();
				vector<Literal>::iterator lend = mergedLits.end();
				uint cl_sz = lend - lbeg;

				void* mem								= malloc(sizeof(Clause) + sizeof(Literal)*cl_sz);
				if(mem == NULL){
					fprintf(stderr,"\nc !!!OUT OF MEMORY - Try to Fix!!!\n");
					exit(EXIT_FAILURE);
					return false;
				}
				Clause* clToAdd 						= new (mem) Clause(cl_sz, false, 0);

				for(uint32_t i = 0;lbeg != lend;++lbeg)		{(*clToAdd)[i++] = *lbeg;}
				(*clToAdd).computeSignature();


				merged.push_back(clToAdd);
				literals += mergedLits.size();
			}
		}
	}

	return literals;
}


/**
 * Method tries to eliminate the variable x from the database due to substitution or clause distribution
 * -> a tricky thing, because the eliminated variable is maybe good for propagation in solver runs
 * returns whether the variable was eliminated or not
 */
bool Simplifier::mayEliminate(Variable x)
{
	asrt(!varEliminated.isSet(x.idx()));

	vector<Clause*>&   poss = occur[x.toPosLit().idx()];
    vector<Clause*>&   negs = occur[x.toNegLit().idx()];
    vector<Clause*>    new_clauses;


    //in occurrence lists are deleted clauses: do not count them:
    uint possSize = 0;
    uint beforeLits = 0;
    for(vector<Clause*>::iterator it = poss.begin();it!=poss.end();it++)
    	if(!(**it).null())
    	{
    		possSize++;
    		beforeLits += (**it).size();
    	}
    int negsSize = 0;
    for(vector<Clause*>::iterator it = negs.begin();it!=negs.end();it++)
      	if(!(**it).null())
       	{
       		negsSize++;
       		beforeLits += (**it).size();
       	}


    //if no positive occurrences or no negative occurrences no need to try elimination
    //TODO: merk dir die Variable! -> wichtig für Solver
    if(possSize == 0 || negsSize == 0)
    	return false;

    //######################	CLAUSE DISTRIBUTION		#######################
    if(possSize >= 8 && negsSize >= 8)
    	return false;

    if(beforeLits >= 100)// && possSize >= 4 && negsSize >= 4)
    	return false;

    new_clauses.clear();
    int resultsize = -1;
    resultsize = mergeClauses(x.toPosLit(), poss, negs, new_clauses);

	if(new_clauses.size() <= possSize + negsSize)
		return addSetOfClausesduringElim(x,new_clauses);
	else
		for(vector<Clause*>::iterator it = new_clauses.begin(); it != new_clauses.end(); it++)
			free(*it);

	return false;
}


/**
 * ( a  b  c  d) is blocked through the clauses
 *
 * (~a  x  y ~d)
 * (~a  s  t ~c)
 * (~a  v  w ~d)
 */
void Simplifier::detectBlockingClauses(uint32_t timeout)
{
//	blockedClauseIDs.resize(0);

	uint lastBlockedClauseDetectedWithLiteral = 0;

	//search for new blocked clauses until no new can be found...
	uint i=1;
	while(lastBlockedClauseDetectedWithLiteral != i)
	{
		if(i % 10 == 0)
		if((*myStats).cpuUserTime() > timeout)
			break;

		if(detectBlockingClauses(Literal(i)))
			lastBlockedClauseDetectedWithLiteral = i;

		i++;
		if(Literal(i).toVar().idx() == varAssignment.size())
			i = 0;
	}

	//print_out("detected %d blocking clauses \n",amountOfBlockingClauses);
}

bool Simplifier::detectBlockingClauses(Literal x)
{

	//for every clause in occur[x] test all clauses in occur[~x]
	vector<Clause*>& poss = occur[x.idx()];
	vector<Clause*>& negs = occur[x.flipped().idx()];

	bool detected = false;

	for(vector<Clause*>::iterator it = poss.begin() ; it != poss.end() ; it++)
	{
		bool isBlocked = true;

		if((**it).null())
			continue;

		vector<Literal> possLits = (**it).getAllLits();

		uint tested = 0;

		for(vector<Clause*>::iterator it2 = negs.begin() ; it2 != negs.end() ; it2++)
		{
			if((**it2).null())
				continue;

			tested++;
			vector<Literal> negsLits = (**it2).getAllLits();
			//"merge" possLits and negsLits, but do not merge x and ~x into the merging vector
			//break if a tautology is found
			bool isTautology = false;

			//use the sortness of both input vectors
			vector<Literal>::iterator posLitIT = possLits.begin();
			vector<Literal>::iterator negLitIT = negsLits.begin();

			while((posLitIT != possLits.end()) || (negLitIT != negsLits.end()))
			{
				//if both have not reached their end
				if((posLitIT != possLits.end()) && (negLitIT != negsLits.end()))
				{
					if((*posLitIT).flipped() == (*negLitIT))
					{
						if((*posLitIT) == x)
						{
							posLitIT++;
							negLitIT++;
							continue;
						}else{
							isTautology = true;
							break;
						}
					}

					if( (*posLitIT) == (*negLitIT) )
					{
						posLitIT++;
						negLitIT++;
					}else if( (*posLitIT) < (*negLitIT) )
					{
						posLitIT++;
					}else
					{
						asrt( (*negLitIT) < (*posLitIT) );
						negLitIT++;
					}

				}else{
					break;
				}
			}

			if(!isTautology)
			{
				isBlocked = false;
				break;
			}
		}

		//add the ID of the blocked
		if(isBlocked && tested > 0)
		{
			blockedClauseIDs.push_back((**it).getID());
			(**it).del();
			decreaseActiveClauses();
			amountOfBlockingClauses++;

			detected = true;
		}
	}

	return detected;
}

/**
 * Main simplification procedure
 */
void Simplifier::simplify(uint32_t timeout)
{
	mytimeout = timeout;

	duringSimp = true;

	//identify blocking clauses
	if(blockingClauses)
		detectBlockingClauses((uint32_t) (mytimeout * 0.5));

	//referring to the paper / minisat implementation: touched = added + strengthened
	touched 		= constraints; asrt(isClauseTouched.size()==constraints.size());
	isClauseTouched.setAll();

	//see pseudo code from SatELite 2005 submission version
	//had to change it a little bit...
	//but the idea is the same
	//1. propagate all units
	//2. strengthen clauses due to self subsumption -> may new units occur -> go to 1.
	//3. delete subsumed clauses
	//4. try to eliminate variables -> go to 1. or 2.
	while(touched.size() > 0)
	{
		vector<Clause*> s1; //set of clauses containing a negated literal of touched + clauses in touched
		vector<Clause*> s0; //set of clauses containing a literal of touched

		s1.reserve(activeClauses);
		s0.reserve(activeClauses);

		//vector for each literal that marks whether its occurrence list was added to S0
		vector<bool> addedLitToS0(occur.size(),false);
		vector<bool> addedClauseToS0(constraints.size(), false);
		vector<bool> addedClauseToS1(constraints.size(), false);

		//fill s0 and s1
		for(uint i=0;i<touched.size();i++)
		{
			Clause& curClause = (*touched[i]);
			if(curClause.null())
				continue;

			for(uint j=0;j<curClause.size();j++)
			{
				//look whether the occurrence lists were already added, do not add it twice
				Literal curLit = curClause[j];

				if(!addedLitToS0[curLit.idx()])
				{
					vector<Clause*>& pOccur = occur[curLit.idx()];
					for(vector<Clause*>::iterator myIT = pOccur.begin(); myIT != pOccur.end(); myIT++)
						if(!(**myIT).null())
							if(!addedClauseToS0[(**myIT).getID()])
							{
								s0.push_back(*myIT);
								addedClauseToS0[(**myIT).getID()] = true;
							}
					vector<Clause*>& nOccur = occur[curLit.flipped().idx()];
					for(vector<Clause*>::iterator myIT = nOccur.begin(); myIT != nOccur.end(); myIT++)
						if(!(**myIT).null())
							if(!addedClauseToS1[(**myIT).getID()])
							{
								s1.push_back(*myIT);
								addedClauseToS1[(**myIT).getID()] = true;
							}
					addedLitToS0[curLit.idx()] = true;
				}
			}
		}

		while(touched.size() > 0)
		{
			//propagation already done when a new unit is found, no need for doing it here...

			for(uint i=0;i<touched.size();i++)
			{
				Clause* curClause = touched[i];
				if(!(*curClause).null())
				{
					if(!addedClauseToS1[(*curClause).getID()])
						s1.push_back(curClause);

				}
			}
			addedClauseToS0 = vector<bool>(constraints.size(), false);
			addedClauseToS1 = vector<bool>(constraints.size(), false);

			touched.clear(); asrt(isClauseTouched.size() == constraints.size());
			isClauseTouched.clearAll();

			//subsume - all kind of stuff
			//may obtain new stuff to propagate

			//strengthen the selfSubsumed clauses
			for(uint i=0;i<s1.size();i++)
			{
				if((*s1[i]).null())
					continue;

				if(!selfSubsumeVariant(s1[i]))
				{
					duringSimp = false;
					return;
				}
				if(i % 10 == 0)
				if((*myStats).cpuUserTime() > mytimeout)
				{
					duringSimp = false;
					return;
				}
			}
			s1.clear();
		}

		//findsubsumedClauses -> eliminate them
		for(uint i=0;i<s0.size();i++)
		{
			Clause* curClause = s0[i];
			if((*curClause).null())
				continue;

			eraseSubsumed(curClause);
			if(i % 10 == 0)
			if((*myStats).cpuUserTime() > mytimeout)
			{
				duringSimp = false;
				return;
			}
		}
		s0.clear();

		//look which variables were touched
		queue<uint32_t> touchedVarIDs;

		for(uint32_t i=1;i<varAssignment.size();i++)
		{
			Variable curVar(i);
			if(addedLitToS0[curVar.toPosLit().idx()] || addedLitToS0[curVar.toNegLit().idx()])
				touchedVarIDs.push(i);
		}

		//TODO: implement a heuristic that sorts the variables according to their likelihood to be eliminated

		while(!touchedVarIDs.empty())
		{
			//try elimination for all touched variables
			uint32_t curID = touchedVarIDs.front();

			bool success=false;
			if(varelim)
			if(varAssignment[curID] == UNKNOWN_TrBool)
			{
				success = mayEliminate(Variable(curID));

				if(success)
				{
//					if(curID == 3739 || curID == 3712)
//						cerr << "var elim " << curID << endl;

					varEliminated.setFlag(curID);
				}
			}
			if(status != UNKNOWN_TrBool)
			{
				duringSimp = false;
				return;
			}

			//propagation already done when a new unit is found, no need for doing it here...
			touchedVarIDs.pop();

			if((*myStats).cpuUserTime() > mytimeout)
			{
				duringSimp = false;
				return;
			}
		}

		if(touched.size() == 0)
		{
			if(blockingClauses)
				detectBlockingClauses(mytimeout);
		}

		if((*myStats).cpuUserTime() > mytimeout)
		{
			duringSimp = false;
			return;
		}
	}

	duringSimp = false;
}


void Simplifier::setSolvedStatus(TrBool myStatus)
{


	status = myStatus;
//	cleanupClauses();

	//prepare for export
	if(myStatus == FALSE_TrBool)
		(*mySolver).setInvalid();


}

void Simplifier::cleanupClauses()
{
	blockedClauseIDs.clear();
	isClauseTouched.dealloc();
	occur.clear();
	touched.clear();

	for(uint i=1;i<constraints.size();i++)
	{
		if(constraints[i] != constraints[0])
			free(constraints[i]);
	}

//	free(constraints[0]);

	constraints.clear();

	activeClauses = 0;
//	addedLitToS0.clear();
//	addedClauseToS0.clear();
//	addedClauseToS1.clear();


}

void Simplifier::compAssignments(vector<TrBool>& solverAssignments)
{
//	//reAdd the deleted constraints
//	for(uint i=0;i<constraints.size();i++)
//	{
//		if(constraints[i] == constraints[0])
//			continue;
//
//		if(!(*constraints[i]).null())
//		{
//			constraints[i] = constraints[0];
//			continue;
//		}else
//			(*constraints[i]).setID(i);
//	}
//
//	//propagate solverAssignments
//	for(uint i=0;i<solverAssignments.size();i++)
//	{
//		if(varAssignment[i] == UNKNOWN_TrBool)
//		{
//			if(solverAssignments[i] == TRUE_TrBool)
//				propagate(Variable(i).toPosLit());
//			else if(solverAssignments[i] == FALSE_TrBool)
//				propagate(Variable(i).toNegLit());
//		}
//	}
//
//	//print assignments
//	printf("v ");
//	for(uint i=1;i<varAssignment.size();i++)
//	{
//		Variable v = Variable(i);
//		TrBool val = varAssignment[i];
////		if(val.isUnknown()) continue;
//
//		printf("%d ",(val.isFalse()?v.toNegLit().toDIMACS():v.toPosLit().toDIMACS()));
//	}
//	printf(" 0\n");

//cerr << "vareliminated[3712]=" << (varEliminated.isSet(3712)?"y":"n") << endl;


	Settings sets = Settings();
		sets.simp = false;
		sets.console_out = NULL;
		sets.hybrid		= false;

		Statistics stats = Statistics(sets);
		DMRPSolver solveit(sets, stats);

		solveit.reserveForClauses(constraints.size()-activeClauses);
		solveit.reserveForVariables(varAssignment.size());

		for(uint i=0;i<varAssignment.size();i++)
		{
			vector<Literal> lits;
			if(varAssignment[i] != UNKNOWN_TrBool)
			{
				if(varAssignment[i] == TRUE_TrBool)
					lits.push_back(Variable(i).toPosLit());
				else
					lits.push_back(Variable(i).toNegLit());
			}else if(!varEliminated.isSet(i))
			{
				if(solverAssignments[i] != UNKNOWN_TrBool){
					if(solverAssignments[i] == TRUE_TrBool){
						lits.push_back(Variable(i).toPosLit());}
					else{
						lits.push_back(Variable(i).toNegLit());}
				}
			}

			if(lits.size() > 0)
			{
//				if(lits[0].toVar().idx() == 3712)
//					cerr << "readd 3712" << endl;

				solveit.addClause(lits);
			}
			lits.clear();
		}

		for(uint i=0;i<constraints.size();i++)
		{
			if(constraints[i] == constraints[0])
				continue;

			if(!(*constraints[i]).null())
				continue;

			vector<Literal> lits;
			lits = (*constraints[i]).getAllLits();
			solveit.addClause(lits);

			lits.clear();
		}

//		cerr << "huhu" << endl;
		solveit.isSAT(~0);
		solveit.printModel();

	cleanupClauses();
}
